# coding=utf-8
from pathlib import Path

import everett
import pytest

from elib.config import ConfigProp


class WrongBaseClass:
    """
    Does not inherit BaseConfig
    """
    string = ConfigProp(str, '')


def test_create_config(dummy_config):
    cfg = dummy_config('test')
    assert cfg.debug is False
    assert not cfg.string
    assert cfg.integer is 0


@pytest.mark.parametrize('key', ['debug', 'DEBUG'])
def test_default_dict(dummy_config, key):
    cfg = dummy_config(
        'test', {key: 'true', 'string': 'some string', 'integer': 1})
    assert cfg.debug is True
    assert cfg.string == 'some string'
    assert cfg.integer is 1


def test_no_default(dummy_config):
    cfg = dummy_config('test')
    with pytest.raises(everett.ConfigurationMissingError):
        assert cfg.no_default == 'value'


def test_wrong_base_class():
    test = WrongBaseClass()
    with pytest.raises(TypeError):
        print(test.string)


def test_calling_from_instance(dummy_config):
    assert isinstance(dummy_config().integer, ConfigProp)


def test_empty_config_file(dummy_config):
    Path('./test.yml').touch()
    cfg = dummy_config('test')
    assert cfg.debug is False



# coding=utf-8
import pytest


@pytest.mark.parametrize('ext', ['yaml', 'yml'])
def test_yaml_config_file(dummy_config, ext):
    with open(f'test.{ext}', 'w') as stream:
        stream.write('''
debug: "true"
string: some string
integer: 1
''')
    cfg = dummy_config('test')
    assert cfg.debug is True
    assert cfg.string == 'some string'
    assert cfg.integer is 1


@pytest.mark.parametrize('ext', ['yaml', 'yml'])
def test_yaml_list_of_str(dummy_config, ext):
    with open(f'test.{ext}', 'w') as stream:
        stream.write('''
debug: "true"
string: some string
integer: 1
some_list:
  - caribou
  - gopher
  - moose
''')
    cfg = dummy_config('test')
    assert cfg.some_list == ['caribou', 'gopher', 'moose']


@pytest.mark.parametrize('ext', ['yaml', 'yml'])
def test_yaml_list_of_int(dummy_config, ext):
    with open(f'test.{ext}', 'w') as stream:
        stream.write('''
debug: "true"
string: some string
integer: 1
some_list:
  - 0
  - -4
  - 3
''')
    cfg = dummy_config('test')
    assert cfg.some_list == [0, -4, 3]


@pytest.mark.parametrize('ext', ['yaml', 'yml'])
def test_yaml_namespace(dummy_config, ext):
    with open(f'test.{ext}', 'w') as stream:
        stream.write('''
debug: "true"
string: "some string"
integer: 1
some_list:
  - caribou
  - gopher
  - moose
namespace:
  key: 'caribou'
''')
    cfg = dummy_config('test')
    assert cfg.namespace_key == 'caribou'



# coding=utf-8
"""
Config property
"""

from .config import BaseConfig


class ConfigProp:
    """
    Decorator-class to create properties for META instances.
    """

    def __init__(self, parser: object, default: object = '__NO_DEFAULT__', namespace: str = None):
        """
        Initialize properties of the descriptor.

        :param default: default value of the property if it isn't set yet
        :param parser:
        """
        self.default = default
        self.parser = parser
        self.namespace = namespace

    @property
    def _name_without_namespace(self):
        if self.namespace is None:
            return self.name

        return self.name.upper().replace(f'{self.namespace.upper()}_', '')

    def _value(self, instance):
        return getattr(instance, '_config')(
            self._name_without_namespace,
            default=self.default,
            parser=self.parser,
            namespace=self.namespace
        )

    def _no_default(self, instance):
        return getattr(instance, '_config')(
            self._name_without_namespace,
            parser=self.parser,
            namespace=self.namespace
        )

    def __get__(self, instance, owner=None):
        """
        Retrieves value.

        If instance is None, returns the descriptor object to allow access to "default" and "type"

        If instance is a valid Config object, returns value from it

        :param instance: instance of AbstractMeta
        :param owner: actual Class of the META instance
        :return: value of this DESCRIPTOR (DEFAULT if it isn't defined)
        """

        # raise FileNotFoundError()
        if instance is None:
            # Calling from Class object
            return self

        if not isinstance(instance, BaseConfig):
            raise TypeError('_ConfigProp can only be used with EverettConfig() instances')

        if self.default == '__NO_DEFAULT__':
            return self._no_default(instance)

        return self._value(instance)

    # pylint: disable=attribute-defined-outside-init
    def __set_name__(self, owner, name):
        self.owner = owner
        self.name = name



# coding=utf-8


def test_ini_config_file(dummy_config):
    with open('test.ini', 'w') as stream:
        stream.write('''
[main]
debug=true
string=some string
integer=1

[namespace]
key=value
''')
    cfg = dummy_config('test')
    assert cfg.debug is True
    assert cfg.string == 'some string'
    assert cfg.integer is 1
    assert cfg.namespace_key == 'value'



# coding=utf-8
"""
Convenience functions to manage logging
"""
from elib.custom_logging._custom_logging_handler import CustomLoggingHandler
from ._custom_logging import get_logger, set_handler_level, get_elib_logger, \
    set_root_logger, get_root_logger



# coding=utf-8

import pytest

from elib.config import BaseConfig, ConfigProp


class DummyConfig(BaseConfig):
    """Dummy test class for Config"""

    debug = ConfigProp(bool, 'false')
    string = ConfigProp(str, '')
    integer = ConfigProp(int, '0')
    some_list = ConfigProp(list, '[]')
    namespace_key = ConfigProp(str, namespace='namespace')
    no_default = ConfigProp(str)


@pytest.fixture()
def dummy_config():
    def make_dummy_config(*args, **kwargs):
        if args or kwargs:
            return DummyConfig(*args, **kwargs)

        return DummyConfig
    yield make_dummy_config



# coding=utf-8


def test_env_config(dummy_config):
    with open(f'.env', 'w') as stream:
        stream.write('''
DEBUG=true
STRING=some string
INTEGER=1
NAMESPACE_KEY=value
''')
    cfg = dummy_config('test')
    print(cfg._config)
    assert cfg.debug is True
    assert cfg.string == 'some string'
    assert cfg.integer is 1
    assert cfg.namespace_key == 'value'
